@@ -19,6 +19,10 @@ gboolean chatty_utils_window_has_toplevel_focus (GtkWindow *window);
 const char *chatty_utils_get_purple_dir (void);
 char *chatty_utils_jabber_id_strip (const char *name);
 void chatty_utils_sanitize_filename (char *name);
+char *chatty_utils_find_url (const char  *buffer,
+                             char       **end);
+char *chatty_utils_strip_utm_from_url (const char *url_to_parse);
+char *chatty_utils_strip_utm_from_message (const char *message);
 char *chatty_utils_check_phonenumber (const char *phone_number,
                                       const char *country);
 ChattyProtocol chatty_utils_username_is_valid  (const char     *name,
